part of 'package:zego_uikit_prebuilt_live_audio_room/src/controller.dart';

/// @nodoc
mixin ZegoLiveAudioRoomControllerMinimizing {
  final _minimizingImpl = ZegoAudioRoomControllerMinimizingImpl();

  ZegoAudioRoomControllerMinimizingImpl get minimize => _minimizingImpl;
}

/// Here are the APIs related to screen sharing.
class ZegoAudioRoomControllerMinimizingImpl
    with ZegoLiveAudioRoomControllerMinimizingPrivate {
  LiveAudioRoomMiniOverlayPageState get state =>
      ZegoUIKitPrebuiltLiveAudioRoomMiniOverlayMachine().state();

  /// Is it currently in the minimized state or not
  bool get isMinimizing =>
      ZegoUIKitPrebuiltLiveAudioRoomMiniOverlayMachine().isMinimizing;

  bool restore(
    BuildContext context, {
    bool rootNavigator = true,
    bool withSafeArea = false,
  }) {
    if (LiveAudioRoomMiniOverlayPageState.minimizing != state) {
      ZegoLoggerService.logInfo(
        'is not minimizing, ignore',
        tag: 'audio room',
        subTag: 'controller.minimize',
      );

      return false;
    }

    final minimizeData = private.minimizeData;
    if (null == minimizeData) {
      ZegoLoggerService.logError(
        'prebuiltData is null',
        tag: 'audio room',
        subTag: 'controller.minimize',
      );

      return false;
    }

    /// re-enter prebuilt call
    ZegoUIKitPrebuiltLiveAudioRoomMiniOverlayMachine().changeState(
      LiveAudioRoomMiniOverlayPageState.inAudioRoom,
    );

    try {
      Navigator.of(context, rootNavigator: rootNavigator).push(
        MaterialPageRoute(builder: (context) {
          return SafeArea(
            child: ZegoUIKitPrebuiltLiveAudioRoom(
              appID: minimizeData.appID,
              appSign: minimizeData.appSign,
              userID: minimizeData.userID,
              userName: minimizeData.userName,
              roomID: minimizeData.roomID,
              config: minimizeData.config,
              controller: minimizeData.controller,
            ),
          );
        }),
      );
    } catch (e) {
      ZegoLoggerService.logError(
        'navigator push to call page exception:$e',
        tag: 'audio room',
        subTag: 'controller.minimize',
      );

      return false;
    }

    return true;
  }

  bool minimize(
    BuildContext context, {
    bool rootNavigator = true,
  }) {
    if (LiveAudioRoomMiniOverlayPageState.minimizing ==
        ZegoUIKitPrebuiltLiveAudioRoomMiniOverlayMachine().state()) {
      ZegoLoggerService.logInfo(
        'is minimizing, ignore',
        tag: 'audio room',
        subTag: 'controller.minimize',
      );

      return false;
    }

    ZegoUIKitPrebuiltLiveAudioRoomMiniOverlayMachine().changeState(
      LiveAudioRoomMiniOverlayPageState.minimizing,
    );

    try {
      /// pop call page
      Navigator.of(
        context,
        rootNavigator: rootNavigator,
      ).pop();
    } catch (e) {
      ZegoLoggerService.logError(
        'navigator pop exception:$e',
        tag: 'audio room',
        subTag: 'controller.minimize',
      );

      return false;
    }

    return true;
  }

  /// if audio room ended in minimizing state, not need to navigate, just hide the minimize widget.
  void hide() {
    ZegoUIKitPrebuiltLiveAudioRoomMiniOverlayMachine().changeState(
      LiveAudioRoomMiniOverlayPageState.idle,
    );
  }
}
